/*
 * entities.h
 *
 * Copyright (c) Chris Putnam 2003-2013
 *
 * Source code released under the GPL version 2
 *
 */
#ifndef ENTITIES_H
#define ENTITIES_H

extern unsigned int decode_entity( char *s, unsigned int *pi, 
		int *unicode, int *err );

#endif

