/*
 * strsearch.h
 *
 * Copyright (c) Chris Putnam 1995-2013
 *
 * Source code released under the GPL version 2
 *
 */
#ifndef STRSEARCH_H
#define STRSEARCH_H

char *strsearch (const char *haystack, const char *needle);

#endif

